package com.fightingchaos.VASSAL.build.module;

import java.awt.AlphaComposite;
import java.awt.Component;
import java.awt.Composite;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;

import com.fightingchaos.VASSAL.counters.Group;
import com.fightingchaos.VASSAL.counters.GroupStackMetrics;

import VASSAL.build.module.Map;
import VASSAL.build.module.documentation.HelpFile;
import VASSAL.build.module.map.StackMetrics;
import VASSAL.counters.GamePiece;
import VASSAL.counters.Properties;
import VASSAL.counters.Stack;


public class GroupingMap extends Map {
	
	
	
	  public StackMetrics getStackMetrics() {
	    if (metrics == null) {
	      metrics = new GroupStackMetrics();
	      metrics.build(null);
	      add(metrics);
	      metrics.addTo(this);
	    }
	    return metrics;
	  }
  
	  
	  public void drawPiecesInRegion(Graphics g, Rectangle visibleRect, Component c) {
		  if (!hideCounters) {
			  Graphics2D g2d = (Graphics2D) g;
			  Composite oldComposite = g2d.getComposite();
			  g2d.setComposite(
					  AlphaComposite.getInstance(AlphaComposite.SRC_OVER, pieceOpacity));
			  GamePiece[] stack = pieces.getPieces();
			  for (int i = 0; i < stack.length; ++i) {
				  Point pt = componentCoordinates(stack[i].getPosition());
				  if (stack[i] instanceof Stack) {
					  getStackMetrics().draw(
							  (Stack) stack[i], pt, g, this, getZoom(), visibleRect);
				  }
				  else {
					  stack[i].draw(g, pt.x, pt.y, c, getZoom());
					  if (Boolean.TRUE.equals(stack[i].getProperty(Properties.SELECTED))) {
						  highlighter.draw(stack[i], g, pt.x, pt.y, c, getZoom());
					  }
				  }
			  }
			  g2d.setComposite(oldComposite);
		  }
	  }

	  /**
	   * Adds a GamePiece to this map. Removes the piece from its parent Stack and from its current map, if different from
	   * this map. Wraps the GamePiece in a Group if the piece is not a stack type
	   */
	  @Override
	  public void addPiece(GamePiece p) {
	    if (indexOf(p) < 0) {
	      if (p.getParent() != null) {
	        p.getParent().remove(p);
	        p.setParent(null);
	      }
	      if (p.getMap() != null && p.getMap() != this) {
	        p.getMap().removePiece(p);
	      }
	      if(p instanceof Stack) {
	    	pieces.add(p);
	    	p.setMap(this);
	      } else {//wrap a group around it for usage on this map
	  		Group grp = new Group();
	  		grp.setPosition(p.getPosition());
	  		grp.setMap(this);
	  		grp.add(p);
	  		pieces.add(grp);
	      }
	      theMap.repaint();
	    }
	  }
	  
	  public HelpFile getHelpFile() {
		    return HelpFile.getReferenceManualPage("com.fightingchaos.VASSAL/index.html", "GroupCommand");
	  }
}
